Option Explicit On
Option Strict On

''' <summary>
''' PreviewForm implements the form that is used to display the visualized data
''' </summary>
''' <remarks></remarks>
Public Class PreviewForm

	''' <summary>
	''' objectType stores the name of the class of the collection being visualized
	''' </summary>
	''' <remarks></remarks>
	Private objectType As String

	''' <summary>
	''' Displays the form
	''' </summary>
	''' <param name="dtData">The DataTable containing the collection data to be visualized</param>
	''' <remarks></remarks>
    Public Sub ShowData(ByVal dtData As DataTable)

        Dim tablestyle As Windows.Forms.DataGridTableStyle

		'Read out the tablename (we used this to identify the object type)
        objectType = dtData.TableName
        tslblInfo.Text = objectType & " contains " & dtData.Rows.Count & " item" & IIf(dtData.Rows.Count = 1, "", "s").ToString
        'Now clear the tablename, otherwise the grid styles don't work
        dtData.TableName = ""

		'Initialise the grid
        With gridData
			.AllowNavigation = False
            .AllowSorting = True
            .ReadOnly = True
            .GridLineColor = Drawing.SystemColors.ControlDark
			.CaptionVisible = False

			'Create a datagridtablestyle to allow us to configure the grid content
			tablestyle = New Windows.Forms.DataGridTableStyle()

			'Determine which columns are present in the DataTable so that we can create corresponding grid columns
            Select Case dtData.Columns.Count
                Case 1
					tablestyle.GridColumnStyles.Add(ShowData_CreateColumn(dtData.Columns("Value"), 622, HorizontalAlignment.Left))
				Case 2
					tablestyle.GridColumnStyles.Add(ShowData_CreateColumn(dtData.Columns("Name"), 200, HorizontalAlignment.Left))
					tablestyle.GridColumnStyles.Add(ShowData_CreateColumn(dtData.Columns("Value"), 422, HorizontalAlignment.Left))
				Case 3
					tablestyle.GridColumnStyles.Add(ShowData_CreateColumn(dtData.Columns("Name"), 200, HorizontalAlignment.Left))
					tablestyle.GridColumnStyles.Add(ShowData_CreateColumn(dtData.Columns("Value"), 302, HorizontalAlignment.Left))
					tablestyle.GridColumnStyles.Add(ShowData_CreateColumn(dtData.Columns("Expires"), 120, HorizontalAlignment.Left))
			End Select

			'Apply the style to the grid
			.TableStyles.Add(tablestyle)

			'Set the grid's DataSource so that our data appears
			.DataSource = dtData

		End With

		'All ready, let's display the form
		Me.ShowDialog()

	End Sub

	''' <summary>
	''' Helper function to quickly create and initialise a column for the DataGrid
	''' </summary>
	''' <param name="Column">The DataColumn from which data is to be displayed</param>
	''' <param name="Width">The width for the new grid column</param>
	''' <param name="Alignment">The text alignment for the new grid column</param>
	''' <returns></returns>
	''' <remarks></remarks>
	Private Function ShowData_CreateColumn(ByVal Column As DataColumn, ByVal Width As Integer, ByVal Alignment As Windows.Forms.HorizontalAlignment) As Windows.Forms.DataGridTextBoxColumn

		Dim col As New CustomGridTextBoxColumn ' Windows.Forms.DataGridTextBoxColumn

		col.MappingName = Column.ColumnName
		col.HeaderText = Column.Caption
		col.Width = Width
		col.Alignment = Alignment

		Return col

	End Function


	''' <summary>
	''' Handle clicking the About button
	''' </summary>
	''' <param name="sender"></param>
	''' <param name="e"></param>
	''' <remarks></remarks>
    Private Sub tsbAbout_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsbAbout.Click

        Dim txt As New System.Text.StringBuilder

        'Get the assembly title
        txt.Append(DirectCast(Reflection.Assembly.GetExecutingAssembly.GetCustomAttributes(GetType(Reflection.AssemblyTitleAttribute), False)(0), Reflection.AssemblyTitleAttribute).Title.ToString)
        'Add the version
        txt.Append(" v" & FileVersionInfo.GetVersionInfo(Me.GetType.Assembly.Location).FileVersion().ToString & vbCrLf)
        txt.Append(vbCrLf)
        txt.Append("Copyright  Adam Dawes, 2007-2008" & vbCrLf)
        txt.Append(vbCrLf)
        txt.Append("I can be contacted at the following address:" & vbCrLf)
        txt.Append("  adam@adamdawes.com" & vbCrLf)
        txt.Append(vbCrLf)
        txt.Append("For updates and other products, visit my web site:" & vbCrLf)
        txt.Append("  www.adamdawes.com")

        MsgBox(txt.ToString, MsgBoxStyle.Information)

    End Sub

	''' <summary>
	''' Handle clicking the Copy button
	''' </summary>
	''' <param name="sender"></param>
	''' <param name="e"></param>
	''' <remarks></remarks>
	Private Sub tsbCopy_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsbCopy.Click

		'Clear the clipboard and then copy in the collection data in Text format
		Clipboard.Clear()
		Clipboard.SetText(GetText())

	End Sub

	''' <summary>
	''' Handle clicking the Save button
	''' </summary>
	''' <param name="sender"></param>
	''' <param name="e"></param>
	''' <remarks></remarks>
    Private Sub tsbSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsbSave.Click

        Dim saveDlg As New Windows.Forms.SaveFileDialog

        With saveDlg
            .AddExtension = True
            .DefaultExt = "jpg"
            .Filter = "CSV files (*.csv)|*.csv|Text files (*.txt)|*.txt|XML files (*.xml)|*.xml"
            .FilterIndex = 0
            .OverwritePrompt = True
            .ShowHelp = False
            .Title = "Save collection data"
            .ValidateNames = True
			If .ShowDialog = Windows.Forms.DialogResult.OK Then
				'Save according to the specified extension
				Select Case LCase(IO.Path.GetExtension(.FileName))
					Case ".csv"
						IO.File.WriteAllText(.FileName, GetCSV())
					Case ".txt"
						IO.File.WriteAllText(.FileName, GetText())
					Case ".xml"
						IO.File.WriteAllText(.FileName, GetXML())
					Case Else
						MsgBox("Unknown save format: " & IO.Path.GetExtension(.FileName), MsgBoxStyle.Exclamation)
				End Select
			End If
        End With

    End Sub

	''' <summary>
	''' Retrieve a textual representation of the data stored within the collection
	''' </summary>
	''' <returns></returns>
	''' <remarks></remarks>
    Private Function GetText() As String

        Dim dtData As DataTable
        Dim drv As DataRowView
        Dim dc As DataColumn
        Dim ret As New System.Text.StringBuilder

		'Retrieve the data from the grid
        dtData = DirectCast(gridData.DataSource, DataTable)

        'Add the column headers
        For Each dc In dtData.Columns
            'If this is not the first column, add a space
            If dc IsNot dtData.Columns(0) Then
                ret.Append(" ")
            End If
            'Add the value
            ret.Append(dc.ColumnName)
        Next
        'End of the line!
        ret.Append(vbCrLf)

        'Add the data
        For Each drv In dtData.DefaultView
            For Each dc In dtData.Columns
                'If this is not the first column, add a space
                If dc IsNot dtData.Columns(0) Then
                    ret.Append(" ")
                End If
                'Add the value
                ret.Append(drv(dc.ColumnName).ToString)
            Next

            'End of the line!
            ret.Append(vbCrLf)
        Next

        Return ret.ToString

    End Function

	''' <summary>
	''' Retrieve a CSV representation of the data stored within the collection
	''' </summary>
	''' <returns></returns>
	''' <remarks></remarks>
    Private Function GetCSV() As String

        Dim dtData As DataTable
        Dim drv As DataRowView
        Dim dc As DataColumn
        Dim ret As New System.Text.StringBuilder
        Dim theValue As String

		'Retrieve the data from the grid
        dtData = DirectCast(gridData.DataSource, DataTable)

        'Add the column headers
        For Each dc In dtData.Columns
            'Get the value, appropriately delimited
            theValue = GetCSV_EncodeValue(dc.ColumnName, ",")
            'If this is not the first column, add the leading delimiter
            If dc IsNot dtData.Columns(0) Then
                ret.Append(",")
            End If
            'Add the value
            ret.Append(theValue)
        Next
        'End of the line!
        ret.Append(vbCrLf)

        'Add the data
        For Each drv In dtData.DefaultView
            For Each dc In dtData.Columns
                'Get the value, appropriately delimited
                theValue = GetCSV_EncodeValue(drv(dc.ColumnName).ToString, ",")

                'If this is not the first column, add the leading delimiter
                If dc IsNot dtData.Columns(0) Then
                    ret.Append(",")
                End If
                'Add the value
                ret.Append(theValue)
            Next

            'End of the line!
            ret.Append(vbCrLf)
        Next

        Return ret.ToString

    End Function

    ''' <summary>
    ''' Encode the supplied value as appropriate for inclusion in the delimited file.
    ''' </summary>
    ''' <param name="TheValue">The value to be encoded</param>
    ''' <param name="Delimiter">The delimiter being used within the file</param>
    ''' <returns>Encoded value</returns>
    ''' <remarks></remarks>
    Private Function GetCSV_EncodeValue(ByVal TheValue As String, ByVal Delimiter As String) As String

        Dim addQuotes As Boolean

        'Does the value contain double-quotes?
        If InStr(TheValue, """") > 0 Then
            'Yes, so double them up and indicate we need to put this value in quotes
            TheValue = Replace(TheValue, """", """""")
            addQuotes = True
        End If

        'Does the value contain the delimiter?
        If InStr(TheValue, Delimiter) > 0 Then
            'Yes, so we will need to add quotes
            addQuotes = True
        End If

		'Replace any line breaks with plain LineFeed characters. This is the requirement for linebreaks in the CSV values
        TheValue = Replace(TheValue, vbCrLf, vbLf)

        'Add the quotes if necessary
        If addQuotes Then
            TheValue = """" & TheValue & """"
        End If

        Return TheValue

    End Function

	''' <summary>
	''' Retrieve a XML representation of the data stored within the collection
	''' </summary>
	''' <returns></returns>
	''' <remarks></remarks>
	Private Function GetXML() As String

		Dim memStream As New IO.MemoryStream
		Dim dtData As DataTable
		Dim sr As IO.StreamReader = Nothing
		Dim ret As String

		Try
			'Retrieve the data from the grid
			dtData = DirectCast(gridData.DataSource, DataTable)

			'Get the DataTable to provide an XML representation of all its data.
			'This is much easier than building the XML manually, but presents a couple
			'of minor issues to work around:
			'The structure of the document will be:
			'
			'	<DocumentElement>
			'		<(TableName)>
			'			<(FieldName)>(FieldValue)</(FieldName)>
			'			[...]
			'		</(TableName)>
			'		[...]
			'	</DocumentElement>
			'
			'(Values in brackets are replaced by the appropriate data from the DataTable).
			'To make the output nicer, we'll temporarily set the TableName to "Item",
			'to ensure that each collection item is contained within an Item XML element.
			dtData.TableName = "Item"
			'Write the XML to a memorystread
			dtData.WriteXml(memStream)
			'Change the TableName back to its original value
			dtData.TableName = ""

			'Retrieve the data from the memorystread
			sr = New IO.StreamReader(memStream)
			'Reset the memory stream back to the start
			memStream.Position = 0
			'Read out all of the memorystream content
			ret = sr.ReadToEnd

			'Finally, we'll replace the top-level DocumentElement element name with the
			'classname of the object being visualized. This could be achieved in a much
			'cleaner fashion by loading it into a proper XML object and changing its name,
			'but for the sake of simplicity we'll just do a simple string replace here
			'instead.
			ret = Replace(ret, "DocumentElement>", objectType & ">")
			Return ret

		Finally
			'Clean up
			If sr IsNot Nothing Then sr.Dispose()
			If memStream IsNot Nothing Then memStream.Dispose()

		End Try

	End Function

End Class